@@ -1,4 +1,4 @@
-{-# LANGUAGE FlexibleContexts, FlexibleInstances, FunctionalDependencies, GeneralizedNewtypeDeriving, MultiParamTypeClasses, RankNTypes, UndecidableInstances  #-}
+{-# LANGUAGE FlexibleContexts, FlexibleInstances, FunctionalDependencies, GeneralizedNewtypeDeriving, MultiParamTypeClasses, RankNTypes, TypeFamilies, UndecidableInstances  #-}
 {-| This module defines the Monad stack used by Happstack. You mostly don't want to be looking in here. Look in "Happstack.Server.Monads" instead.
 -}
 module Happstack.Server.Internal.Monads where
@@ -7,23 +7,32 @@ import Control.Applicative                       (Applicative, pure, (<*>), Alte
 
 import Control.Monad                             ( MonadPlus(mzero, mplus), ap, liftM, msum
                                                  )
-import Control.Monad.Trans                       ( MonadTrans, lift
-                                                 , MonadIO, liftIO
-                                                 )
-import Control.Monad.Reader                      ( ReaderT(ReaderT), runReaderT
-                                                 , MonadReader, ask, local
-                                                 )
-import Control.Monad.Writer                      ( WriterT(WriterT), runWriterT
-                                                 , MonadWriter, tell, pass
-                                                 , listens
-                                                 )
-import qualified Control.Monad.Writer            as Writer (listen) -- So that we can disambiguate 'Listen.listen'
-import Control.Monad.State                       (MonadState, get, put)
+import Control.Monad.Base                        ( MonadBase, liftBase )
 import Control.Monad.Error                       ( ErrorT(ErrorT), runErrorT
                                                  , Error, MonadError, throwError
                                                  , catchError, mapErrorT
                                                  )
-import Control.Monad.Maybe                       (MaybeT(MaybeT), runMaybeT)
+import Control.Monad.Reader                      ( ReaderT(ReaderT), runReaderT
+                                                 , MonadReader, ask, local, mapReaderT
+                                                 )
+import Control.Monad.RWS                         ( RWST, mapRWST
+                                                 )
+
+import Control.Monad.State                       (MonadState, StateT, get, put, mapStateT)
+import Control.Monad.Trans                       ( MonadTrans, lift
+                                                 , MonadIO, liftIO
+                                                 )
+import Control.Monad.Trans.Control               ( MonadTransControl(..)
+                                                 , MonadBaseControl(..)
+                                                 , ComposeSt, defaultLiftBaseWith, defaultRestoreM
+                                                 )
+import Control.Monad.Writer                      ( WriterT(WriterT), runWriterT
+                                                 , MonadWriter, tell, pass
+                                                 , listens, mapWriterT
+                                                 )
+import qualified Control.Monad.Writer            as Writer (listen) -- So that we can disambiguate 'Listen.listen'
+
+import Control.Monad.Trans.Maybe                 (MaybeT(MaybeT), runMaybeT)
 import qualified Data.ByteString.Lazy.UTF8       as LU (fromString)
 import Data.Char                                 (ord)
 import Data.List                                 (inits, isPrefixOf, stripPrefix, tails)
@@ -48,10 +57,25 @@ type ServerPart a = ServerPartT IO a
 newtype ServerPartT m a = ServerPartT { unServerPartT :: ReaderT Request (WebT m) a }
     deriving (Monad, MonadPlus, Functor)
 
+instance MonadBase b m => MonadBase b (ServerPartT m) where
+    liftBase = lift . liftBase
+
 instance (MonadIO m) => MonadIO (ServerPartT m) where
     liftIO = ServerPartT . liftIO
     {-# INLINE liftIO #-}
 
+instance MonadTransControl ServerPartT where
+    newtype StT ServerPartT a = StSP {unStSP :: StT WebT (StT (ReaderT Request) a)}
+    liftWith f = ServerPartT $ liftWith $ \runReader ->
+                                 liftWith $ \runWeb ->
+                                   f $ liftM StSP . runWeb . runReader . unServerPartT
+    restoreT = ServerPartT . restoreT . restoreT . liftM unStSP
+
+instance MonadBaseControl b m => MonadBaseControl b (ServerPartT m) where
+    newtype StM (ServerPartT m) a = StMSP {unStMSP :: ComposeSt ServerPartT m a}
+    liftBaseWith = defaultLiftBaseWith StMSP
+    restoreM     = defaultRestoreM     unStMSP
+
 -- | Particularly useful when combined with 'runWebT' to produce
 -- a @m ('Maybe' 'Response')@ from a 'Request'.
 runServerPartT :: ServerPartT m a -> Request -> WebT m a
@@ -228,12 +252,25 @@ filterFun :: (a -> a) -> FilterFun a
 filterFun = Set . Dual . Endo
 
 newtype FilterT a m b = FilterT { unFilterT :: WriterT (FilterFun a) m b }
-   deriving (Monad, MonadTrans, Functor)
+   deriving (Functor, Applicative, Monad, MonadTrans)
+
+instance MonadBase b m => MonadBase b (FilterT a m) where
+    liftBase = lift . liftBase
 
 instance (MonadIO m) => MonadIO (FilterT a m) where
     liftIO = FilterT . liftIO
     {-# INLINE liftIO #-}
 
+instance MonadTransControl (FilterT a) where
+    newtype StT (FilterT a) b = StFilter {unStFilter :: StT (WriterT (FilterFun a)) b}
+    liftWith f = FilterT $ liftWith $ \run -> f $ liftM StFilter . run . unFilterT
+    restoreT = FilterT . restoreT . liftM unStFilter
+
+instance MonadBaseControl b m => MonadBaseControl b (FilterT a m) where
+    newtype StM (FilterT a m) c = StMFilter {unStMFilter :: ComposeSt (FilterT a) m c}
+    liftBaseWith = defaultLiftBaseWith StMFilter
+    restoreM     = defaultRestoreM     unStMFilter
+
 -- | A set of functions for manipulating filters.  
 --
 -- 'ServerPartT' implements 'FilterMonad' 'Response' so these methods
@@ -270,10 +307,30 @@ instance (Monad m) => FilterMonad a (FilterT a m) where
 newtype WebT m a = WebT { unWebT :: ErrorT Response (FilterT (Response) (MaybeT m)) a }
     deriving (Functor)
 
+instance MonadBase b m => MonadBase b (WebT m) where
+    liftBase = lift . liftBase
+
 instance (MonadIO m) => MonadIO (WebT m) where
     liftIO = WebT . liftIO
     {-# INLINE liftIO #-}
 
+instance MonadTransControl WebT where
+    newtype StT WebT a = StWeb {unStWeb :: StT MaybeT
+                                             (StT (FilterT Response)
+                                               (StT (ErrorT Response) a))}
+    liftWith f = WebT $ liftWith $ \runError ->
+                          liftWith $ \runFilter ->
+                            liftWith $ \runMaybe ->
+                              f $ liftM StWeb . runMaybe .
+                                                  runFilter .
+                                                    runError . unWebT
+    restoreT = WebT . restoreT . restoreT . restoreT . liftM unStWeb
+
+instance MonadBaseControl b m => MonadBaseControl b (WebT m) where
+    newtype StM (WebT m) a = StMWeb {unStMWeb :: ComposeSt WebT m a}
+    liftBaseWith = defaultLiftBaseWith StMWeb
+    restoreM     = defaultRestoreM     unStMWeb
+
 -- | 'UnWebT' is almost exclusively used with 'mapServerPartT'. If you
 -- are not using 'mapServerPartT' then you do not need to wrap your
 -- head around this type. If you are -- the type is not as complex as
@@ -501,3 +558,71 @@ escapeString str = concatMap encodeEntity str
       encodeEntity c
           | ord c > 127 = "&#" ++ show (ord c) ++ ";"
           | otherwise = [c]
+
+------------------------------------------------------------------------------
+-- ServerMonad, FilterMonad, and WebMonad instances for ReaderT, StateT, 
+-- WriterT, and RWST
+------------------------------------------------------------------------------
+
+-- ReaderT
+
+instance (ServerMonad m) => ServerMonad (ReaderT r m) where
+    askRq         = lift askRq
+    localRq f     = mapReaderT (localRq f)
+
+instance (FilterMonad res m) => FilterMonad res (ReaderT r m) where
+    setFilter f   = lift $ setFilter f
+    composeFilter = lift . composeFilter
+    getFilter     = mapReaderT getFilter
+
+instance (WebMonad a m) => WebMonad a (ReaderT r m) where
+    finishWith    = lift . finishWith
+
+-- StateT
+
+instance (ServerMonad m) => ServerMonad (StateT s m) where
+    askRq         = lift askRq
+    localRq f     = mapStateT (localRq f)
+
+instance (FilterMonad res m) => FilterMonad res (StateT s m) where
+    setFilter f   = lift $ setFilter f
+    composeFilter = lift . composeFilter
+    getFilter   m = mapStateT (\m' -> 
+                                   do ((b,s), f) <- getFilter m'
+                                      return ((b, f), s)) m
+
+instance (WebMonad a m) => WebMonad a (StateT s m) where
+    finishWith    = lift . finishWith
+
+
+-- WriterT
+
+instance (ServerMonad m, Monoid w) => ServerMonad (WriterT w m) where
+    askRq         = lift askRq
+    localRq f     = mapWriterT (localRq f)
+
+instance (FilterMonad res m, Monoid w) => FilterMonad res (WriterT w m) where
+    setFilter f   = lift $ setFilter f
+    composeFilter = lift . composeFilter
+    getFilter   m = mapWriterT (\m' -> 
+                                   do ((b,w), f) <- getFilter m'
+                                      return ((b, f), w)) m
+
+instance (WebMonad a m, Monoid w) => WebMonad a (WriterT w m) where
+    finishWith    = lift . finishWith
+
+-- RWST
+
+instance (ServerMonad m, Monoid w) => ServerMonad (RWST r w s m) where
+    askRq         = lift askRq
+    localRq f     = mapRWST (localRq f)
+
+instance (FilterMonad res m, Monoid w) => FilterMonad res (RWST r w s m) where
+    setFilter f   = lift $ setFilter f
+    composeFilter = lift . composeFilter
+    getFilter   m = mapRWST (\m' -> 
+                                   do ((b,s,w), f) <- getFilter m'
+                                      return ((b, f), s, w)) m
+
+instance (WebMonad a m, Monoid w) => WebMonad a (RWST r w s m) where
+    finishWith     = lift . finishWith
