@@ -4,31 +4,34 @@
 -}
 module Happstack.Server.Internal.TimeoutSocket where
 
-import           Control.Concurrent            (ThreadId, forkIO, killThread, threadDelay, threadWaitWrite)
-import           Control.Exception             (SomeException, catch, throw)
+import           Control.Concurrent            (threadWaitWrite)
+import           Control.Exception             (catch, throw)
 import           Control.Monad                 (liftM, when)
 import qualified Data.ByteString.Char8         as B
 import qualified Data.ByteString.Lazy.Char8    as L
 import qualified Data.ByteString.Lazy.Internal as L
 import qualified Data.ByteString               as S
-import qualified Network.Socket.ByteString as N
-import           Data.Word
-import           Data.IORef
-import           Data.List (foldl')
-import           Data.Time.Clock.POSIX(POSIXTime, getPOSIXTime)
-import           Happstack.Server.Internal.Clock (getApproximatePOSIXTime)
+import           Network.Socket                (sClose)
+import qualified Network.Socket.ByteString     as N
 import qualified Happstack.Server.Internal.TimeoutManager as TM
+import           Happstack.Server.Internal.TimeoutIO (TimeoutIO(..))
 import           Network.Socket (Socket, ShutdownCmd(..), shutdown)
 import           Network.Socket.SendFile (Iter(..), ByteCount, Offset, sendFileIterWith')
 import           Network.Socket.ByteString (sendAll)
 import           Prelude hiding (catch)
-import           System.IO (Handle, hClose, hIsEOF, hWaitForInput)
 import           System.IO.Error (isDoesNotExistError)
 import           System.IO.Unsafe (unsafeInterleaveIO)
 
-sPutTickle :: TM.Handle -> Socket -> L.ByteString -> IO ()
-sPutTickle thandle sock cs =
+sPutLazyTickle :: TM.Handle -> Socket -> L.ByteString -> IO ()
+sPutLazyTickle thandle sock cs =
     do L.foldrChunks (\c rest -> sendAll sock c >> TM.tickle thandle >> rest) (return ()) cs
+{-# INLINE sPutLazyTickle #-}
+
+sPutTickle :: TM.Handle -> Socket -> B.ByteString -> IO ()
+sPutTickle thandle sock cs =
+    do sendAll sock cs
+       TM.tickle thandle
+       return ()
 {-# INLINE sPutTickle #-}
 
 sGetContents :: TM.Handle 
@@ -64,3 +67,14 @@ iterTickle thandle =
                       iterTickle' cont
                (Sent _ cont) ->
                    do iterTickle' cont
+
+timeoutSocketIO :: TM.Handle -> Socket -> TimeoutIO
+timeoutSocketIO handle socket =
+    TimeoutIO { toHandle      = handle
+              , toShutdown    = sClose socket 
+              , toPutLazy     = sPutLazyTickle handle socket
+              , toPut         = sPutTickle     handle socket
+              , toGetContents = sGetContents   handle socket
+              , toSendFile    = sendFileTickle handle socket
+              , toSecure      = False
+              }
