@@ -1,4 +1,4 @@
-{-# LANGUAGE FlexibleContexts, TypeSynonymInstances, ScopedTypeVariables #-}
+{-# LANGUAGE FlexibleContexts, FlexibleInstances, TypeSynonymInstances, ScopedTypeVariables #-}
 -- | Functions and classes related to generating a 'Response' and setting the response code. For detailed instruction see the Happstack Crash Course: <http://happstack.com/docs/crashcourse/HelloWorld.html#response_code>
 module Happstack.Server.Response 
     ( -- * Converting values to a 'Response'
@@ -14,6 +14,8 @@ module Happstack.Server.Response
     , unauthorized
     , forbidden
     , notFound
+    , prettyResponse
+    , requestEntityTooLarge
     , seeOther
     , found
     , movedPermanently
@@ -32,7 +34,7 @@ import qualified Data.Text                       as T
 import qualified Data.Text.Encoding              as T
 import qualified Data.Text.Lazy                  as LT
 import qualified Data.Text.Lazy.Encoding         as LT
-import           Happstack.Server.Monads         (FilterMonad(composeFilter))
+import           Happstack.Server.Internal.Monads         (FilterMonad(composeFilter))
 import           Happstack.Server.Types          (Response(..), Request(..), nullRsFlags, getHeader, noContentLength, redirect, result, setHeader, setHeaderBS)
 import           Happstack.Server.SURI           (ToSURI)
 import           System.Locale                   (defaultTimeLocale)
@@ -68,7 +70,7 @@ toResponseBS contentType message =
 -- | 'toResponse' will convert a value into a 'Response' body,
 -- set the @content-type@, and set the default response code for that type.
 --
--- Example:
+-- @happstack-server@ Example:
 --
 -- > main = simpleHTTP nullConf $ toResponse "hello, world!"
 --
@@ -79,6 +81,10 @@ toResponseBS contentType message =
 --
 --  > main = simpleHTTP nullConf $ "hello, world!"
 --
+-- @happstack-lite@ Example:
+--
+-- > main = serve Nothing $ toResponse "hello, world!"
+--
 -- Minimal definition: 'toMessage' (and usually 'toContentType'). 
 class ToMessage a where
     toContentType :: a -> B.ByteString
@@ -136,6 +142,13 @@ instance ToMessage Blaze.Html where
 
 instance ToMessage Response where
     toResponse = id
+
+instance ToMessage L.ByteString where
+    toResponse bs = Response 200 M.empty nullRsFlags bs Nothing
+
+instance ToMessage B.ByteString where
+    toResponse bs = toResponse (L.fromChunks [bs])
+
 {-
 
 -- This instances causes awful error messages. I am removing it and
@@ -218,17 +231,39 @@ ok = resp 200
 noContent :: (FilterMonad Response m) => a -> m a
 noContent val = composeFilter (\r -> noContentLength (r { rsCode = 204, rsBody = L.empty })) >> return val
 
--- | Respond with @500 Internal Server Error@.
+-- | Respond with @301 Moved Permanently@.
 --
--- > main = simpleHTTP nullConf $ internalServerError "Sorry, there was an internal server error."
-internalServerError :: (FilterMonad Response m) => a -> m a
-internalServerError = resp 500
+-- > main = simpleHTTP nullConf $ movedPermanently "http://example.org/" "What you are looking for is now at http://example.org/"
+movedPermanently :: (FilterMonad Response m, ToSURI a) => a -> res -> m res
+movedPermanently uri res = do modifyResponse $ redirect 301 uri
+                              return res
 
--- | Responds with @502 Bad Gateway@.
+-- | Respond with @302 Found@.
+-- 
+-- You probably want 'seeOther'. This method is not in popular use anymore, and is generally treated like 303 by most user-agents anyway.
+found :: (FilterMonad Response m, ToSURI uri) => uri -> res -> m res
+found uri res = do modifyResponse $ redirect 302 uri
+                   return res
+
+-- | Respond with @303 See Other@.
 --
--- > main = simpleHTTP nullConf $ badGateway "Bad Gateway."
-badGateway :: (FilterMonad Response m) => a -> m a
-badGateway = resp 502
+-- > main = simpleHTTP nullConf $ seeOther "http://example.org/" "What you are looking for is now at http://example.org/"
+--
+-- NOTE: The second argument of 'seeOther' is the message body which will sent to the browser. According to the HTTP 1.1 spec,
+--
+-- @the entity of the response SHOULD contain a short hypertext note with a hyperlink to the new URI(s).@
+--
+-- This is because pre-HTTP\/1.1 user agents do not support 303. However, in practice you can probably just use @\"\"@ as the second argument.
+seeOther :: (FilterMonad Response m, ToSURI uri) => uri -> res -> m res
+seeOther uri res = do modifyResponse $ redirect 303 uri
+                      return res
+
+-- | Respond with @307 Temporary Redirect@.
+--
+-- > main = simpleHTTP nullConf $ tempRedirect "http://example.org/" "What you are looking for is temporarily at http://example.org/"
+tempRedirect :: (FilterMonad Response m, ToSURI a) => a -> res -> m res
+tempRedirect val res = do modifyResponse $ redirect 307 val
+                          return res
 
 -- | Respond with @400 Bad Request@.
 --
@@ -254,36 +289,39 @@ forbidden = resp 403
 notFound :: (FilterMonad Response m) => a -> m a
 notFound = resp 404
 
--- | Respond with @303 See Other@.
+-- | Respond with @413 Request Entity Too Large@.
 --
--- > main = simpleHTTP nullConf $ seeOther "http://example.org/" "What you are looking for is now at http://example.org/"
---
--- NOTE: The second argument of 'seeOther' is the message body which will sent to the browser. According to the HTTP 1.1 spec,
---
--- @the entity of the response SHOULD contain a short hypertext note with a hyperlink to the new URI(s).@
---
--- This is because pre-HTTP\/1.1 user agents do not support 303. However, in practice you can probably just use @\"\"@ as the second argument.
-seeOther :: (FilterMonad Response m, ToSURI uri) => uri -> res -> m res
-seeOther uri res = do modifyResponse $ redirect 303 uri
-                      return res
+-- > main = simpleHTTP nullConf $ requestEntityTooLarge "That's too big for me to handle."
+requestEntityTooLarge :: (FilterMonad Response m) => a -> m a
+requestEntityTooLarge = resp 413
 
--- | Respond with @302 Found@.
--- 
--- You probably want 'seeOther'. This method is not in popular use anymore, and is generally treated like 303 by most user-agents anyway.
-found :: (FilterMonad Response m, ToSURI uri) => uri -> res -> m res
-found uri res = do modifyResponse $ redirect 302 uri
-                   return res
-
--- | Respond with @301 Moved Permanently@.
+-- | Respond with @500 Internal Server Error@.
 --
--- > main = simpleHTTP nullConf $ movedPermanently "http://example.org/" "What you are looking for is now at http://example.org/"
-movedPermanently :: (FilterMonad Response m, ToSURI a) => a -> res -> m res
-movedPermanently uri res = do modifyResponse $ redirect 301 uri
-                              return res
+-- > main = simpleHTTP nullConf $ internalServerError "Sorry, there was an internal server error."
+internalServerError :: (FilterMonad Response m) => a -> m a
+internalServerError = resp 500
 
--- | Respond with @307 Temporary Redirect@.
+-- | Responds with @502 Bad Gateway@.
 --
--- > main = simpleHTTP nullConf $ tempRedirect "http://example.org/" "What you are looking for is temporarily at http://example.org/"
-tempRedirect :: (FilterMonad Response m, ToSURI a) => a -> res -> m res
-tempRedirect val res = do modifyResponse $ redirect 307 val
-                          return res
+-- > main = simpleHTTP nullConf $ badGateway "Bad Gateway."
+badGateway :: (FilterMonad Response m) => a -> m a
+badGateway = resp 502
+
+-- | A nicely formatted rendering of a 'Response'
+prettyResponse :: Response -> String
+prettyResponse res@Response{}  =
+    showString   "================== Response ================" .
+    showString "\nrsCode      = " . shows      (rsCode res)     .
+    showString "\nrsHeaders   = " . shows      (rsHeaders res)  .
+    showString "\nrsFlags     = " . shows      (rsFlags res)    .
+    showString "\nrsBody      = " . shows      (rsBody res)     .
+    showString "\nrsValidator = " $ show       (rsValidator res)
+prettyResponse res@SendFile{}  =
+    showString   "================== Response ================" .
+    showString "\nrsCode      = " . shows      (rsCode res)     .
+    showString "\nrsHeaders   = " . shows      (rsHeaders res)  .
+    showString "\nrsFlags     = " . shows      (rsFlags res)    .
+    showString "\nrsValidator = " . shows      (rsValidator res).
+    showString "\nsfFilePath  = " . shows      (sfFilePath res) .
+    showString "\nsfOffset    = " . shows      (sfOffset res)   .
+    showString "\nsfCount     = " $ show       (sfCount res)    
