@@ -1,9 +1,12 @@
-{-# LANGUAGE TypeSynonymInstances, DeriveDataTypeable #-}
+{-# LANGUAGE DeriveDataTypeable, FlexibleInstances, TypeSynonymInstances #-}
 -- | A wrapper and type class so that functions like 'seeOther' can take a URI which is represented by a 'String', 'URI.URI', or other instance of 'ToSURI'. 
 module Happstack.Server.SURI where
+
+import Control.Arrow (first)
 import Data.Maybe
 import Data.Generics
-import Happstack.Util.Common(mapFst)
+import qualified Data.Text as Text
+import qualified Data.Text.Lazy as LazyText
 import qualified Network.URI as URI
 
 -- | Retrieves the path component from the URI
@@ -34,8 +37,9 @@ a_scheme a (SURI u) = SURI $ u {URI.uriScheme=a}
 a_path :: String -> SURI -> SURI
 a_path a (SURI u) = SURI $ u {URI.uriPath=a}
 
-escape, unEscape :: String -> String
-unEscape = URI.unEscapeString . map (\x->if x=='+' then ' ' else x)
+escape, unEscape, unEscapeQS :: String -> String
+unEscapeQS = URI.unEscapeString . map (\x->if x=='+' then ' ' else x)
+unEscape = URI.unEscapeString
 escape = URI.escapeURIString URI.isAllowedInURI
 
 -- | Returns true if the URI is absolute
@@ -47,6 +51,9 @@ instance Show SURI where
     showsPrec d (SURI uri) = showsPrec d $ show uri
 instance Read SURI where
     readsPrec d = mapFst fromJust .  filter (isJust . fst) . mapFst parse . readsPrec d 
+      where
+        mapFst :: (a -> b) -> [(a,x)] -> [(b,x)]
+        mapFst = map . first
 
 instance Ord SURI where
     compare a b = show a `compare` show b
@@ -66,7 +73,8 @@ instance ToSURI SURI where toSURI=id
 instance ToSURI URI.URI where toSURI=SURI
 instance ToSURI String where 
     toSURI = maybe (SURI $ URI.URI "" Nothing "" "" "") id . parse
-
+instance ToSURI Text.Text where toSURI = toSURI . Text.unpack
+instance ToSURI LazyText.Text where toSURI = toSURI . LazyText.unpack
 
 --handling obtaining things from URI paths
 class FromPath x where fromPath::String->x
