@@ -6,14 +6,14 @@ import qualified Codec.Compression.Zlib as Z
 import Control.Arrow ((&&&))
 import Control.Applicative ((<$>))
 import Control.Concurrent.MVar
-import Control.Monad (msum)
+import Control.Monad (msum, forM_)
 import Control.Monad.Trans (liftIO)
 import Data.ByteString.Lazy.Char8     (pack, unpack)
-import qualified Data.ByteString.Char8 as B 
+import qualified Data.ByteString.Char8 as B
 import qualified Data.ByteString.Lazy  as L
 import qualified Data.Map              as Map
 import Happstack.Server                      ( Request(..), Method(..), Response(..), ServerPart(..), Headers(..), RqBody(Body), HttpVersion(..)
-                                             , ToMessage(..), HeaderPair(..), ok, dir, simpleHTTP'', composeFilter, noContentLength)
+                                             , ToMessage(..), HeaderPair(..), ok, dir, simpleHTTP'', composeFilter, noContentLength, matchMethod)
 import Happstack.Server.FileServe.BuildingBlocks (sendFileResponse)
 import Happstack.Server.Cookie
 import Happstack.Server.Internal.Compression
@@ -24,17 +24,18 @@ import Happstack.Server.SURI(ToSURI(..), SURI(..),path,query)
 import Test.HUnit as HU (Test(..),(~:),(~?),(@?=),(@=?), assertEqual, assertFailure)
 import Text.ParserCombinators.Parsec
 
--- |All of the tests for happstack-util should be listed here. 
+-- |All of the tests for happstack-util should be listed here.
 allTests :: Test
-allTests = 
+allTests =
     "happstack-server tests" ~: [ cookieParserTest
                                 , acceptEncodingParserTest
                                 , multipart
                                 , compressFilterResponseTest
+                                , matchMethodTest
                                 ]
 
 cookieParserTest :: Test
-cookieParserTest = 
+cookieParserTest =
     "cookieParserTest" ~:
     [parseCookies "$Version=1;Cookie1=value1;$Path=\"/testpath\";$Domain=example.com;cookie2=value2"
         @?= (Right [
@@ -107,8 +108,8 @@ multipart =
 
 compressFilterResponseTest :: Test
 compressFilterResponseTest =
-    "compressFilterResponseTest" ~: 
-     [ uncompressedResponse 
+    "compressFilterResponseTest" ~:
+     [ uncompressedResponse
      , uncompressedSendFile
      , compressedResponseGZ
      , compressedResponseZ
@@ -117,9 +118,9 @@ compressFilterResponseTest =
      ]
 
 mkRequest :: Method -> String -> [(String, Cookie)] -> Headers -> L.ByteString -> IO Request
-mkRequest method uri cookies headers body = 
+mkRequest method uri cookies headers body =
     do let u = toSURI uri
-       ib <- newEmptyMVar 
+       ib <- newEmptyMVar
        b  <- newMVar (Body body)
        return $ Request { rqMethod      = method
                         , rqPaths       = (pathEls (path u))
@@ -196,3 +197,27 @@ compressedSendFileNoIdentity =
          assertEqual "respone code"     (rsCode res) 406
          assertEqual "body"             (unpack (rsBody res)) ""
          assertEqual "Content-Encoding" ((hName &&& hValue) <$> Map.lookup (B.pack "content-encoding") (rsHeaders res)) Nothing
+
+matchMethodTest :: Test
+matchMethodTest =
+    "matchMethodTest" ~:
+      do forM_ gethead $ \m -> matchMethod GET m @?= True
+         forM_ others  $ \m -> matchMethod GET m @?= False
+         forM_ gethead $ \m -> matchMethod [GET] m @?= True
+         forM_ others  $ \m -> matchMethod [GET] m @?= False
+         forM_ gethead $ \m -> matchMethod [GET, HEAD] m @?= True
+         forM_ others  $ \m -> matchMethod [GET, HEAD] m @?= False
+         matchMethod POST GET   @?= False
+         matchMethod POST HEAD  @?= False
+         matchMethod POST TRACE @?= False
+         matchMethod POST POST  @?= True
+         matchMethod [POST, PUT] GET   @?= False
+         matchMethod [POST, PUT] HEAD  @?= False
+         matchMethod [POST, PUT] TRACE @?= False
+         matchMethod [POST, PUT] POST  @?= True
+         matchMethod [POST, PUT] PUT   @?= True
+         forM_ (others) $ \m -> matchMethod (`notElem` gethead) m @?= True
+         forM_ (gethead ++ others) $ \m -> matchMethod () m @?= True
+  where
+    gethead = [GET, HEAD]
+    others  = [POST, PUT, DELETE, TRACE, OPTIONS, CONNECT]
